import pickle
import numpy as np
import load_saved_models as lsm
import time
import tqdm
import pandas as pd
import os


DATA_PATH = os.path.abspath(os.path.join(__file__, '../../results/regression'))


def get_bests(var_names = ['Mn(kg·mol)-1', 'MwMn', 'conversion']):
    bests = {}
    for var_name in var_names:
        bests[var_name] = {}
        all = pd.read_excel(os.path.join(DATA_PATH, f'{var_name}_mean.xlsx'), index_col=0)
        for column in all.columns[0::2]:
            results = all[column]
            indexes = np.array(list(all.index))
            order = np.argsort(results.values)[::-1]
            bests[var_name][column.replace('(mean)', '')] = list(indexes[order[:3]])
            
    return bests

def save_bests_excel(var_name, data):
    # data in format LR: {'RAW_PCA'}
    print(data)
    excel_data = {'Preprocessing': [f"{l}{k}".replace(var_name, '') for k in data for l in data[k]],
                  'MAE': [data[k][l][1] for k in data for l in data[k]],
                  'Model': ['LR', 'LR', 'LR', 'SVR', 'SVR', 'SVR', 'RF', 'RF', 'RF']}    
    
    df = pd.DataFrame(excel_data)
    df.set_index('Preprocessing', inplace=True)
    df.to_excel(f'{var_name}_bests.xlsx')


if __name__ == "__main__":

    var_names = ['Mn (kg·mol)-1', 'MwMn', 'conversion']
    dict_model = {}
    bests_dict = {}
    seeds = [645, 4578, 72, 2365, 90345, 24, 1859, 1334, 2078, 2446, 7409, 6995, 2041, 449, 9475]
    
    all_bests = get_bests()

    # Iterate over all the best models and create datasets
    for var in var_names:
        bests = all_bests[var.replace(' ', '')]
        bests_dict[var] = {}
        for best_key in bests:
            bests_dict[var][best_key] = {}
            for name in bests[best_key]:
                print(var, best_key, name)    
                yhat_val = []
                maes_val = []
                mapes_val = []
                model_varname = f'{name}_{var}'
                model_type = 'linear' if best_key == 'LR' else ('forest' if best_key == 'RF' else 'svr')

                for seed in tqdm.tqdm(seeds): 
                    data, ensemble = lsm.load_ensemble(f'{name}_{var}', f'./results/models/{seed}/{name}_{var}_{model_type}.pkl', seed)
                    y_hat = ensemble.predict_ensemble(data[0])
                    mae, mape = ensemble.compute_mae_mape_ensemble(data[0], data[1])
                    yhat_val.append(y_hat)
                    maes_val.append(mae)
                    mapes_val.append(mape)

                bests_dict[var][best_key][model_varname] = [np.mean(yhat_val), np.mean(maes_val), np.mean(mapes_val)]
                
        save_bests_excel(var, bests_dict[var])

    pickle.dump(bests_dict, open('all_bests.pkl', 'wb'))
    
    # Iterate over all the best RF models and save the data
    best_rf = [all_bests[v.replace(' ','')]['RF'] for v in var_names]
    
    for var, best_model in zip(var_names, best_rf):
        for name in best_model:    
            yhat_val = []
            maes_val = []
            mapes_val = []
            model_varname = f'{name}_{var}'
            time_t = []
            
            for seed in tqdm.tqdm(seeds): 
                data, ensemble = lsm.load_ensemble(f'{name}_{var}', f'./results/models/{seed}/{name}_{var}_forest.pkl',seed)
                start_time = time.time()
                y_hat = ensemble.predict_ensemble(data[0])
                end_time = time.time()
                execution_time = end_time - start_time
                time_t.append(execution_time)
                mae, mape = ensemble.compute_mae_mape(data[0], data[1])
                yhat_val.append(y_hat)
                maes_val.append(mae)
                mapes_val.append(mape)

            dict_model[model_varname] = [np.mean(yhat_val, axis = 0),np.mean(maes_val, axis = 0),np.mean(mapes_val, axis = 0),np.mean(time_t)]
        
        pickle.dump([dict_model],open('resolution_%s.pkl' %(var), 'wb'))