import os
import pickle
import regex as re
import pandas as pd
from typing import Any


class DataProvider:
    def __init__(self) -> None:
        self.data = {} # Formato: Nombre: Dict({nombre_variable: [x, y]})
    
    @property
    def ids(self):
        return list(self.data.keys())
    
    def get_id_regex(self, regex):
        names = []
        for id in self.ids:
            res = re.findall(regex, id)
            if res:
                names.append(id)

        return names
        
    def set_data(self, name:str, data: Any):
        self.data[name] = data
        
    def get_data(self, name):
        return self.data[name]
    
    def to_pickle(self, path = './data/all_processed.pkl'):
        with open(path, 'wb') as f:
            pickle.dump(self.data, f)


class Results:
    def __init__(self, seed = None) -> None:
        
        var_names = ['Mn (kg·mol)-1', 'MwMn', 'conversion']
        
        rname = ['RAW','RAW_PCA_var','RAW_PCA_sam','RAW_PCA_poly_var','RAW_PCA_poly_sam','RAW_PCA_rbf_var','RAW_PCA_rbf_sam','RAW_XGB', 'RAW_PCA_var_XGB','RAW_PCA_sam_XGB','RAW_PCA_poly_var_XGB','RAW_PCA_poly_sam_XGB','RAW_PCA_rbf_var_XGB','RAW_PCA_rbf_sam_XGB', 'RAW_RF','RAW_PCA_var_RF', 'RAW_PCA_sam_RF','RAW_PCA_poly_var_RF','RAW_PCA_poly_sam_RF','RAW_PCA_rbf_var_RF','RAW_PCA_rbf_sam_RF']
        
        cname = ['LR(mean)', 'LR(std)', 'SVR(mean)','SVR(std)','RF(mean)','RF(std)']
        self.seed = seed
        self.results = {}
        self.models = {}
        
        for name in var_names:
            self.results[name] = pd.DataFrame(index=rname, columns=cname)
            
    def set_value(self, var_name, row_name, col_name, value):
        self.results[var_name].at[row_name, col_name] = value
    
    def set_models(self, name, models):
        self.models[name] = models
    
    def retrieve_data(self, var_name):
        return self.results[var_name]
    
    def to_csv(self, path = './results/regression/'):
        for k in self.results:
            n = f"{path}{k.replace(' ', '').replace('/', '_')}"
            n += f'_{self.seed}.csv' if self.seed is not None else ''

            self.results[k].to_csv(n)
    
    def to_excel(self, path = './results/regression/'):
        for k in self.results:
            n = f"{path}{k.replace(' ', '').replace('/', '_')}"
            n += f'_{self.seed}.xlsx' if self.seed is not None else ''
            
            self.results[k].to_excel(n)
    
    def save_models(self, path='./results/models/'):
        os.makedirs(path, exist_ok=True)
        for k in self.models:
            with open(f'{path}{k}.pkl','wb') as f:
                pickle.dump(self.models[k], f)
                f.close()
    